//==========================================================================
//
//      jns200_nand_flash.c
//
//      Flash programming for Toshiba NAND FLASH device on TS7250 board
//
//==========================================================================
//####ECOSGPLCOPYRIGHTBEGIN####
// -------------------------------------------
// This file is part of eCos, the Embedded Configurable Operating System.
// Copyright (C) 1998, 1999, 2000, 2001, 2002 Red Hat, Inc.
// Copyright (C) 2003 Nick Garnett <nickg@calivar.com>
// Copyright (C) 2003, 2004 Gary Thomas
//
// eCos is free software; you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free
// Software Foundation; either version 2 or (at your option) any later version.
//
// eCos is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
// for more details.
//
// You should have received a copy of the GNU General Public License along
// with eCos; if not, write to the Free Software Foundation, Inc.,
// 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.
//
// As a special exception, if other files instantiate templates or use macros
// or inline functions from this file, or you compile this file and link it
// with other works to produce a work based on this file, this file does not
// by itself cause the resulting work to be covered by the GNU General Public
// License. However the source code for this file must still be made available
// in accordance with section (3) of the GNU General Public License.
//
// This exception does not invalidate any other reasons why a work based on
// this file might be covered by the GNU General Public License.
//
// Alternative licenses for eCos may be arranged by contacting Red Hat, Inc.
// at http://sources.redhat.com/ecos/ecos-license/
// -------------------------------------------
//####ECOSGPLCOPYRIGHTEND####
//==========================================================================
//#####DESCRIPTIONBEGIN####
//
// Author(s):    Jesse Off <joff@embeddedARM.com>
// Contributors: 
// Date:         2004-11-09
// Purpose:      
// Description:  
//              
//####DESCRIPTIONEND####
//
//==========================================================================

#include <cyg/infra/cyg_type.h>
#define CYGARC_HAL_COMMON_EXPORT_CPU_MACROS
#include <cyg/hal/hal_io.h>             // I/O macros
#include <cyg/hal/hal_if.h>             // Virtual vector interfaces

// There's a single chip on the TS7250 board. 

#define CYGNUM_FLASH_BLANK	(1)
#define CYGNUM_FLASH_INTERLEAVE	(1)
#define CYGNUM_FLASH_WIDTH      (8)
#define CYGNUM_FLASH_SERIES	(1)
#define CYGNUM_FLASH_BASE 	(0x60000000)

#define CYGHWR_DEVS_FLASH_SAMSUNG_K9F5608U0B
#define CYGHWR_FLASH_K9F56XXX0B_CE   jns200_CE 
#define CYGHWR_FLASH_K9F56XXX0B_CLE  jns200_CLE 
#define CYGHWR_FLASH_K9F56XXX0B_ALE  jns200_ALE
#define CYGHWR_FLASH_K9F56XXX0B_RDY  jns200_RDY
#define CYGHWR_K9F56XXX0B_COMPATIBLE

//
// Select the NAND device - this will be held until the
// entire command/data sequence has completed (which is
// necessary for read sequential)
//
static void __inline__
jns200_CE(int state)
{
    cyg_uint8 ctrl;
    HAL_READ_UINT8((CYGNUM_FLASH_BASE + 0x400000), ctrl);
    if (state) {
        // Assert CE
        ctrl |= 0x4;
    } else {
        // De-assert CE
        ctrl &= ~0x4;
    }
    HAL_WRITE_UINT8((CYGNUM_FLASH_BASE + 0x400000), ctrl);
}

//
// Prepare to send a command byte to the NAND interface
//
static void __inline__
jns200_CLE(int state)
{
    cyg_uint8 ctrl;
    HAL_READ_UINT8((CYGNUM_FLASH_BASE + 0x400000), ctrl);
    if (state) {
        // Assert CLE
        ctrl |= 0x2;
    } else {
        // De-assert CLE
        ctrl &= ~0x2;
    }
    HAL_WRITE_UINT8((CYGNUM_FLASH_BASE + 0x400000), ctrl);
}

//
// Prepare to send address byte(s) to the NAND interface
//
static void __inline__
jns200_ALE(int state)
{
    cyg_uint8 ctrl;

    HAL_READ_UINT8((CYGNUM_FLASH_BASE + 0x400000), ctrl);
    if (state) {
        // Assert ALE
        ctrl |= 0x1;
    } else {
        // De-assert ALE
        ctrl &= ~0x1;
    }
    HAL_WRITE_UINT8((CYGNUM_FLASH_BASE + 0x400000), ctrl);
}

//
// Wait until the NAND device is ready (after a long operation)
//
static bool __inline__
jns200_RDY(void)
{    
    cyg_uint8 status;

    HAL_READ_UINT8((CYGNUM_FLASH_BASE + 0x800000), status);
    return ((status & 0x20) != 0);
}

#include <cyg/io/flash_k9f56xxx0b.inl>

// ------------------------------------------------------------------------
// EOF jns200_nand_flash.c
