//==========================================================================
//
//      ep93xx_misc.c
//
//      HAL misc board support code for ARM9/EP93XX
//
//==========================================================================
//####COPYRIGHTBEGIN####
//
// -------------------------------------------
// The contents of this file are subject to the Red Hat eCos Public License
// Version 1.1 (the "License"); you may not use this file except in
// compliance with the License.  You may obtain a copy of the License at
// http://www.redhat.com/
//
// Software distributed under the License is distributed on an "AS IS"
// basis, WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the
// License for the specific language governing rights and limitations under
// the License. 
//
// The Original Code is eCos - Embedded Configurable Operating System,
// released September 30, 1998.
//
// The Initial Developer of the Original Code is Red Hat.
// Portions created by Red Hat are
// Copyright (C) 1998, 1999, 2000, 2001 Red Hat, Inc.
// All Rights Reserved.
// -------------------------------------------
//
//####COPYRIGHTEND####
//==========================================================================
//#####DESCRIPTIONBEGIN####
//
// Author(s):    gthomas
// Contributors: hmt, Travis C. Furrer <furrer@mit.edu>, jskov
// Date:         2001-04-23
// Purpose:      HAL board support
// Description:  Implementations of HAL board interfaces
//
//####DESCRIPTIONEND####
//
//========================================================================*/
#include <pkgconf/hal.h>
#include <pkgconf/system.h>
#include CYGBLD_HAL_PLATFORM_H

#include <cyg/infra/cyg_type.h>         // base types
#include <cyg/infra/cyg_trac.h>         // tracing macros
#include <cyg/infra/cyg_ass.h>          // assertion macros

#include <cyg/hal/hal_io.h>             // IO macros
#include <cyg/hal/hal_arch.h>           // Register state info
#include <cyg/hal/hal_diag.h>
#include <cyg/hal/hal_intr.h>           // Interrupt names
#include <cyg/hal/hal_cache.h>
#include <cyg/hal/ep93xx.h>             // Platform specifics
#include <cyg/hal/ts7250.h>             // Platform specifics
#include <cyg/hal/regs_VIC.h>

#include <cyg/infra/diag.h>             // diag_printf

// -------------------------------------------------------------------------
// MMU initialization:
// 
// These structures are laid down in memory to define the translation
// table.
// 

/*
 * SA-1100 Translation Table Base Bit Masks */
#define ARM_TRANSLATION_TABLE_MASK               0xFFFFC000

/*
 * SA-1100 Domain Access Control Bit Masks
 */
#define ARM_ACCESS_TYPE_NO_ACCESS(domain_num)    (0x0 << (domain_num)*2)
#define ARM_ACCESS_TYPE_CLIENT(domain_num)       (0x1 << (domain_num)*2)
#define ARM_ACCESS_TYPE_MANAGER(domain_num)      (0x3 << (domain_num)*2)

struct ARM_MMU_FIRST_LEVEL_FAULT {
    int id : 2;
    int sbz : 30;
};
#define ARM_MMU_FIRST_LEVEL_FAULT_ID 0x0

struct ARM_MMU_FIRST_LEVEL_PAGE_TABLE {
    int id : 2;
    int imp : 2;
    int domain : 4;
    int sbz : 1;
    int base_address : 23;
};
#define ARM_MMU_FIRST_LEVEL_PAGE_TABLE_ID 0x1

struct ARM_MMU_FIRST_LEVEL_SECTION {
    int id : 2;
    int b : 1;
    int c : 1;
    int imp : 1;
    int domain : 4;
    int sbz0 : 1;
    int ap : 2;
    int sbz1 : 8;
    int base_address : 12;
};
#define ARM_MMU_FIRST_LEVEL_SECTION_ID 0x2

struct ARM_MMU_FIRST_LEVEL_RESERVED {
    int id : 2;
    int sbz : 30;
};
#define ARM_MMU_FIRST_LEVEL_RESERVED_ID 0x3

#define ARM_MMU_FIRST_LEVEL_DESCRIPTOR_ADDRESS(ttb_base, table_index) \
   (unsigned long *)((unsigned long)(ttb_base) + ((table_index) << 2))

#define ARM_FIRST_LEVEL_PAGE_TABLE_SIZE 0x4000

#define ARM_MMU_SECTION(ttb_base, actual_base, virtual_base,              \
                        cacheable, bufferable, perm)                      \
    CYG_MACRO_START                                                       \
        register union ARM_MMU_FIRST_LEVEL_DESCRIPTOR desc;               \
                                                                          \
        desc.word = 0;                                                    \
        desc.section.id = ARM_MMU_FIRST_LEVEL_SECTION_ID;                 \
        desc.section.imp = 1;                                             \
        desc.section.domain = 0;                                          \
        desc.section.c = (cacheable);                                     \
        desc.section.b = (bufferable);                                    \
        desc.section.ap = (perm);                                         \
        desc.section.base_address = (actual_base);                        \
        *ARM_MMU_FIRST_LEVEL_DESCRIPTOR_ADDRESS(ttb_base, (virtual_base)) \
                            = desc.word;                                  \
    CYG_MACRO_END

#define X_ARM_MMU_SECTION(abase,vbase,size,cache,buff,access)      \
    { int i; int j = abase; int k = vbase;                         \
      for (i = size; i > 0 ; i--,j++,k++)                          \
      {                                                            \
        ARM_MMU_SECTION(ttb_base, j, k, cache, buff, access);      \
      }                                                            \
    }

union ARM_MMU_FIRST_LEVEL_DESCRIPTOR {
    unsigned long word;
    struct ARM_MMU_FIRST_LEVEL_FAULT fault;
    struct ARM_MMU_FIRST_LEVEL_PAGE_TABLE page_table;
    struct ARM_MMU_FIRST_LEVEL_SECTION section;
    struct ARM_MMU_FIRST_LEVEL_RESERVED reserved;
};

#define ARM_UNCACHEABLE                         0
#define ARM_CACHEABLE                           1
#define ARM_UNBUFFERABLE                        0
#define ARM_BUFFERABLE                          1

#define ARM_ACCESS_PERM_NONE_NONE               0
#define ARM_ACCESS_PERM_RO_NONE                 0
#define ARM_ACCESS_PERM_RO_RO                   0
#define ARM_ACCESS_PERM_RW_NONE                 1
#define ARM_ACCESS_PERM_RW_RO                   2
#define ARM_ACCESS_PERM_RW_RW                   3

extern int memset( void *, int, size_t );

void
hal_mmu_init(void)
{
    unsigned long ttb_base = EP9312_SDRAM_PHYS_BASE + 0x4000;
    unsigned long i;

    /*
     * Set the TTB register
     */
    asm volatile ("mcr  p15,0,%0,c2,c0,0" : : "r"(ttb_base) /*:*/);

    /*
     * Set the Domain Access Control Register
     */
    i = ARM_ACCESS_TYPE_MANAGER(0)    | 
        ARM_ACCESS_TYPE_NO_ACCESS(1)  |
        ARM_ACCESS_TYPE_NO_ACCESS(2)  |
        ARM_ACCESS_TYPE_NO_ACCESS(3)  |
        ARM_ACCESS_TYPE_NO_ACCESS(4)  |
        ARM_ACCESS_TYPE_NO_ACCESS(5)  |
        ARM_ACCESS_TYPE_NO_ACCESS(6)  |
        ARM_ACCESS_TYPE_NO_ACCESS(7)  |
        ARM_ACCESS_TYPE_NO_ACCESS(8)  |
        ARM_ACCESS_TYPE_NO_ACCESS(9)  |
        ARM_ACCESS_TYPE_NO_ACCESS(10) |
        ARM_ACCESS_TYPE_NO_ACCESS(11) |
        ARM_ACCESS_TYPE_NO_ACCESS(12) |
        ARM_ACCESS_TYPE_NO_ACCESS(13) |
        ARM_ACCESS_TYPE_NO_ACCESS(14) |
        ARM_ACCESS_TYPE_NO_ACCESS(15);
    asm volatile ("mcr  p15,0,%0,c3,c0,0" : : "r"(i) /*:*/);

    /*
     * First clear all Page Table entries - ie Set them to Faulting
     */
    memset((void *)ttb_base, 0, ARM_FIRST_LEVEL_PAGE_TABLE_SIZE);

	// TS7250 Memory Map
    //               Actual  Virtual Size   Attributes
    //		           Base     Base   MB   cached?          buffered?        access permissions
    //             xxx00000  xxx00000

	//
	// SDRAM is physically located in 8MB blocks at 0x00000000, 0x01000000, 0x04000000, 0x05000000
	//
    X_ARM_MMU_SECTION(0x000,  0x000,   8,  ARM_CACHEABLE,   ARM_BUFFERABLE,   ARM_ACCESS_PERM_RW_RW);
    X_ARM_MMU_SECTION(0x010,  0x008,   8,  ARM_CACHEABLE,   ARM_BUFFERABLE,   ARM_ACCESS_PERM_RW_RW);
    X_ARM_MMU_SECTION(0x040,  0x010,   8,  ARM_CACHEABLE,   ARM_BUFFERABLE,   ARM_ACCESS_PERM_RW_RW);
    X_ARM_MMU_SECTION(0x050,  0x018,   8,  ARM_CACHEABLE,   ARM_BUFFERABLE,   ARM_ACCESS_PERM_RW_RW);
    /*
    X_ARM_MMU_SECTION(0xe00,  0x020,   8,  ARM_CACHEABLE,   ARM_BUFFERABLE,   ARM_ACCESS_PERM_RW_RW);
    X_ARM_MMU_SECTION(0xe10,  0x028,   8,  ARM_CACHEABLE,   ARM_BUFFERABLE,   ARM_ACCESS_PERM_RW_RW);
    X_ARM_MMU_SECTION(0xe40,  0x030,   8,  ARM_CACHEABLE,   ARM_BUFFERABLE,   ARM_ACCESS_PERM_RW_RW);
    X_ARM_MMU_SECTION(0xe50,  0x038,   8,  ARM_CACHEABLE,   ARM_BUFFERABLE,   ARM_ACCESS_PERM_RW_RW);
    X_ARM_MMU_SECTION(0xc00,  0x040,   8,  ARM_CACHEABLE,   ARM_BUFFERABLE,   ARM_ACCESS_PERM_RW_RW);
    X_ARM_MMU_SECTION(0xc10,  0x048,   8,  ARM_CACHEABLE,   ARM_BUFFERABLE,   ARM_ACCESS_PERM_RW_RW);
    X_ARM_MMU_SECTION(0xc40,  0x050,   8,  ARM_CACHEABLE,   ARM_BUFFERABLE,   ARM_ACCESS_PERM_RW_RW);
    X_ARM_MMU_SECTION(0xc50,  0x058,   8,  ARM_CACHEABLE,   ARM_BUFFERABLE,   ARM_ACCESS_PERM_RW_RW);
    X_ARM_MMU_SECTION(0xd00,  0x060,   8,  ARM_CACHEABLE,   ARM_BUFFERABLE,   ARM_ACCESS_PERM_RW_RW);
    X_ARM_MMU_SECTION(0xd10,  0x068,   8,  ARM_CACHEABLE,   ARM_BUFFERABLE,   ARM_ACCESS_PERM_RW_RW);
    X_ARM_MMU_SECTION(0xd40,  0x070,   8,  ARM_CACHEABLE,   ARM_BUFFERABLE,   ARM_ACCESS_PERM_RW_RW);
    X_ARM_MMU_SECTION(0xd50,  0x078,   8,  ARM_CACHEABLE,   ARM_BUFFERABLE,   ARM_ACCESS_PERM_RW_RW);
    X_ARM_MMU_SECTION(0x020,  0x080,   8,  ARM_CACHEABLE,   ARM_BUFFERABLE,   ARM_ACCESS_PERM_RW_RW);
    X_ARM_MMU_SECTION(0x030,  0x088,   8,  ARM_CACHEABLE,   ARM_BUFFERABLE,   ARM_ACCESS_PERM_RW_RW);
    X_ARM_MMU_SECTION(0x060,  0x090,   8,  ARM_CACHEABLE,   ARM_BUFFERABLE,   ARM_ACCESS_PERM_RW_RW);
    X_ARM_MMU_SECTION(0x070,  0x098,   8,  ARM_CACHEABLE,   ARM_BUFFERABLE,   ARM_ACCESS_PERM_RW_RW);
    X_ARM_MMU_SECTION(0xe20,  0x0a0,   8,  ARM_CACHEABLE,   ARM_BUFFERABLE,   ARM_ACCESS_PERM_RW_RW);
    X_ARM_MMU_SECTION(0xe30,  0x0a8,   8,  ARM_CACHEABLE,   ARM_BUFFERABLE,   ARM_ACCESS_PERM_RW_RW);
    X_ARM_MMU_SECTION(0xe60,  0x0b0,   8,  ARM_CACHEABLE,   ARM_BUFFERABLE,   ARM_ACCESS_PERM_RW_RW);
    X_ARM_MMU_SECTION(0xe70,  0x0b8,   8,  ARM_CACHEABLE,   ARM_BUFFERABLE,   ARM_ACCESS_PERM_RW_RW);
    */

	//
	// There is a 8-16MB Intel Flash on this board at 0x60000000
	//
    X_ARM_MMU_SECTION(0x600,  0x600,   16, ARM_UNCACHEABLE, ARM_UNBUFFERABLE,   ARM_ACCESS_PERM_RW_RW);
    X_ARM_MMU_SECTION(0x700,  0x700,   16, ARM_UNCACHEABLE, ARM_UNBUFFERABLE,   ARM_ACCESS_PERM_RW_RW);

	//
	// The EP9301 registers are located at 0x80000000 in a 256MB block
	//
    X_ARM_MMU_SECTION(0x800,  0x800,   256,ARM_UNCACHEABLE, ARM_UNBUFFERABLE, ARM_ACCESS_PERM_RW_RW);

	//
	// The TS-7200 PC/104 bus and internal peripherals
	//
    X_ARM_MMU_SECTION(0x100,  0x100,   64,ARM_UNCACHEABLE, ARM_UNBUFFERABLE, ARM_ACCESS_PERM_RW_RW);
    X_ARM_MMU_SECTION(0x200,  0x200,   64,ARM_UNCACHEABLE, ARM_UNBUFFERABLE, ARM_ACCESS_PERM_RW_RW);

	//
	// This is an uncached/unbuffered mapping of the SDRAM
	//
    X_ARM_MMU_SECTION(0x000,  0xC00,   8,  ARM_UNCACHEABLE, ARM_UNBUFFERABLE, ARM_ACCESS_PERM_RW_RW);
    X_ARM_MMU_SECTION(0x010,  0xC08,   8,  ARM_UNCACHEABLE, ARM_UNBUFFERABLE, ARM_ACCESS_PERM_RW_RW);
    X_ARM_MMU_SECTION(0x040,  0xC10,   8,  ARM_UNCACHEABLE, ARM_UNBUFFERABLE, ARM_ACCESS_PERM_RW_RW);
    X_ARM_MMU_SECTION(0x050,  0xC18,   8,  ARM_UNCACHEABLE, ARM_UNBUFFERABLE, ARM_ACCESS_PERM_RW_RW);
    X_ARM_MMU_SECTION(0xe00,  0xC20,   8,  ARM_UNCACHEABLE, ARM_UNBUFFERABLE, ARM_ACCESS_PERM_RW_RW);
    X_ARM_MMU_SECTION(0xe10,  0xC28,   8,  ARM_UNCACHEABLE, ARM_UNBUFFERABLE, ARM_ACCESS_PERM_RW_RW);
    X_ARM_MMU_SECTION(0xe40,  0xC30,   8,  ARM_UNCACHEABLE, ARM_UNBUFFERABLE, ARM_ACCESS_PERM_RW_RW);
    X_ARM_MMU_SECTION(0xe50,  0xC38,   8,  ARM_UNCACHEABLE, ARM_UNBUFFERABLE, ARM_ACCESS_PERM_RW_RW);
    /*
    X_ARM_MMU_SECTION(0xc00,  0xc40,   8,  ARM_UNCACHEABLE, ARM_UNBUFFERABLE, ARM_ACCESS_PERM_RW_RW);
    X_ARM_MMU_SECTION(0xc10,  0xc48,   8,  ARM_UNCACHEABLE, ARM_UNBUFFERABLE, ARM_ACCESS_PERM_RW_RW);
    X_ARM_MMU_SECTION(0xc40,  0xc50,   8,  ARM_UNCACHEABLE, ARM_UNBUFFERABLE, ARM_ACCESS_PERM_RW_RW);
    X_ARM_MMU_SECTION(0xc50,  0xc58,   8,  ARM_UNCACHEABLE, ARM_UNBUFFERABLE, ARM_ACCESS_PERM_RW_RW);
    X_ARM_MMU_SECTION(0xd00,  0xc60,   8,  ARM_UNCACHEABLE, ARM_UNBUFFERABLE, ARM_ACCESS_PERM_RW_RW);
    X_ARM_MMU_SECTION(0xd10,  0xc68,   8,  ARM_UNCACHEABLE, ARM_UNBUFFERABLE, ARM_ACCESS_PERM_RW_RW);
    X_ARM_MMU_SECTION(0xd40,  0xc70,   8,  ARM_UNCACHEABLE, ARM_UNBUFFERABLE, ARM_ACCESS_PERM_RW_RW);
    X_ARM_MMU_SECTION(0xd50,  0xc78,   8,  ARM_UNCACHEABLE, ARM_UNBUFFERABLE, ARM_ACCESS_PERM_RW_RW);
    */
    X_ARM_MMU_SECTION(0x020,  0xc80,   8,  ARM_UNCACHEABLE, ARM_UNBUFFERABLE, ARM_ACCESS_PERM_RW_RW);
    X_ARM_MMU_SECTION(0x030,  0xc88,   8,  ARM_UNCACHEABLE, ARM_UNBUFFERABLE, ARM_ACCESS_PERM_RW_RW);
    X_ARM_MMU_SECTION(0x060,  0xc90,   8,  ARM_UNCACHEABLE, ARM_UNBUFFERABLE, ARM_ACCESS_PERM_RW_RW);
    X_ARM_MMU_SECTION(0x070,  0xc98,   8,  ARM_UNCACHEABLE, ARM_UNBUFFERABLE, ARM_ACCESS_PERM_RW_RW);
    X_ARM_MMU_SECTION(0xe20,  0xca0,   8,  ARM_UNCACHEABLE, ARM_UNBUFFERABLE, ARM_ACCESS_PERM_RW_RW);
    X_ARM_MMU_SECTION(0xe30,  0xca8,   8,  ARM_UNCACHEABLE, ARM_UNBUFFERABLE, ARM_ACCESS_PERM_RW_RW);
    X_ARM_MMU_SECTION(0xe60,  0xcb0,   8,  ARM_UNCACHEABLE, ARM_UNBUFFERABLE, ARM_ACCESS_PERM_RW_RW);
    X_ARM_MMU_SECTION(0xe70,  0xcb8,   8,  ARM_UNCACHEABLE, ARM_UNBUFFERABLE, ARM_ACCESS_PERM_RW_RW);


	//
	// This is an uncached/unbuffered mapping of the FLASH
	//
    X_ARM_MMU_SECTION(0x600,  0xE00,   16, ARM_UNCACHEABLE, ARM_UNBUFFERABLE, ARM_ACCESS_PERM_RW_RW);

    // Enable MMU
    asm volatile (
        "mcr	p15,0,%0,c1,c0,0;"
        "nop;nop;nop;nop;nop;nop;nop;nop;"
        :
        : "r"(MMU_Control_Init|MMU_Control_M)
        /* : */ );

}

//=============================================================================
// hal_virt_to_phys_address
//=============================================================================
// Returns a physical address given a virtual address.
//
cyg_uint32 hal_virt_to_phys_address(cyg_uint32 vaddr)
{
    cyg_uint32 paddr;

    //
    // If the Virtual Address two physical mapping is as follows for TS-7250
	// Virtual		Physical
    // --------------------	---------------------
    // 0x00000000-0x007fffff	0x00000000-0x007fffff
    // 0x00800000-0x00ffffff	0x01000000-0x017fffff
    // 0x01000000-0x017fffff	0x04000000-0x047fffff
    // 0x01800000-0x01ffffff	0x05000000-0x057fffff
	//
    // 0xc0000000-0xc07fffff	0x00000000-0x007fffff
    // 0xc0800000-0xc0ffffff	0x01000000-0x017fffff
    // 0xc1000000-0xc17fffff	0x04000000-0x047fffff
    // 0xc1800000-0xc1ffffff	0x05000000-0x057fffff
    //
	
	// 
	// If the memory is in SDRAM then it is mapped to the physical memory at 
	// 0x00000000 in the strange way that memory is mapped.
	//
    if ((((0xC00u * SZ_1M) <= vaddr) && (vaddr <= (0xC20u * SZ_1M ))) ||
	    (((0x000u * SZ_1M) <= vaddr) && (vaddr <= (0x020u * SZ_1M ))))
    {
		vaddr &= 0x0fffffff;
		
		if (vaddr < 0x007fffff)
		{
			paddr = vaddr;
		}
		else if (vaddr < 0x00ffffff)
		{
			paddr = vaddr + (0x008u * SZ_1M);
		}
		else if (vaddr < 0x017fffff)
		{
			paddr = vaddr + (0x030u * SZ_1M);
		}
		else if (vaddr < 0x01ffffff)
		{
			paddr = vaddr + (0x038u * SZ_1M);
		}
    }
	else if (((0xe00u * SZ_1M) <= vaddr) && (vaddr <= (0xe20u * SZ_1M )))
	{
		//
		// Physical address of the Flash is at 0x60000000
		//
        paddr = vaddr - 0x80000000;
	}
    else
    {
		//
		// Otherwise virtual == physical
		//
        paddr = vaddr;
    }

    return(paddr);
}

cyg_uint32 hal_virt_to_uncached_address(cyg_uint32 vaddr)
{
    if ( vaddr <= (0x020u * SZ_1M ))
	{
        vaddr += (0xC00u * SZ_1M);
	}
	else
	{
		vaddr = 0xBAD0ADD8u; /* nothing there; uncached unavailable */
	}

    return (vaddr);
}

cyg_uint32 hal_phys_to_virt_address(cyg_uint32 paddr)		
{
    if ( ((0x010u * SZ_1M) <= paddr) && (paddr < (0x018u * SZ_1M )))
	{
		return (paddr - ((0x010u - 0x008u)* SZ_1M ));
	}
    if ( ((0x040u * SZ_1M) <= paddr) && (paddr < (0x048u * SZ_1M )))
	{
		return (paddr - ((0x040u - 0x010u)* SZ_1M ));
	}
    if ( ((0x050u * SZ_1M) <= paddr) && (paddr < (0x058u * SZ_1M )))
	{
		return (paddr - ((0x050u - 0x018u)* SZ_1M ));
	}
	
	return (paddr);
}

//=============================================================================
//=============================================================================
// Write a locked syscon Register.
static __inline__ void
sysconWriteReg(int offset, unsigned long val)
{
    HAL_WRITE_UINT32(EP9312_SYSCON+EP9312_SYSCON_LOCK, 0xAA);
    HAL_WRITE_UINT32(EP9312_SYSCON+offset, val);
}

//
// Platform specific initialization
//
void
plf_hardware_init(void)
{
    hal_mmu_init();
}

//-----------------------------------------------------------------------------
// This will get defined to HAL_CLOCK_INITIALIZE in hal_intr.h. This routine
// should only be called by test code.
//-----------------------------------------------------------------------------
void hal_clock_initialize(cyg_uint32 period)
{
    while(1);
}

//-----------------------------------------------------------------------------
// This will get defined to HAL_CLOCK_RESET.  This routine should only be
// called by test code.
//-----------------------------------------------------------------------------
externC void hal_clock_reset(cyg_uint32 vector, cyg_uint32 period)
{
    while(1);
}

// Read the current value of the clock, returning the number of hardware
// "ticks" that have occurred (i.e. how far away the current value is from
// the start)

// Note: The "contract" for this function is that the value is the number
// of hardware clocks that have happened since the last interrupt (i.e.
// when it was reset).  This value is used to measure interrupt latencies.
// However, since the hardware counter does not automatically reset, as
// many do, the value may be larger than the system "tick" interval.
// If this turns out to be the case, the result will be adjusted to be
// within the interval [0..N-1], where N is the number of hardware clocks/tick.

void hal_clock_read(cyg_uint32 *pvalue)
{
    while(1);
}

//
// Delay for some number of micro-seconds
//
void hal_delay_us(cyg_int32 usecs)
{
    cyg_uint32 val, val2, hi, hi2;
    int i, safety;
    
    // 
    // Reset debug timer
    //
    HAL_WRITE_UINT32(EP9312_TIMERS_DEBUG_HI, EP9312_TIMERS_DEBUG_HI_RESET);

    // TODO is this required?
    // Required for timer to properly reset??
    //
    for (i = 0;  i < 100;  i++) ;  

    HAL_WRITE_UINT32(EP9312_TIMERS_DEBUG_HI, EP9312_TIMERS_DEBUG_HI_START);

    // 
    // Wait for timer
    //
    do {
        HAL_READ_UINT32(EP9312_TIMERS_DEBUG_LO, val);
        HAL_READ_UINT32(EP9312_TIMERS_DEBUG_HI, hi);
        safety = 1000;  // Surely this is more than 1us
        while (safety-- > 0) {
            HAL_READ_UINT32(EP9312_TIMERS_DEBUG_LO, val2);
            HAL_READ_UINT32(EP9312_TIMERS_DEBUG_HI, hi2);
            if (val != val2) {
                // Timer is going, continue
                break;
            }
        }
        if (safety == 0) {
            // Timer must be stuck, give up
            return;
        }
    } while (val < usecs);
}

//-----------------------------------------------------------------------------
// int hal_IRQ_handler(void)
//-----------------------------------------------------------------------------
// This routine is called to respond to a hardware interrupt (IRQ).  It
// should interrogate the hardware and return the IRQ vector number.
int hal_IRQ_handler(void)
{
    return(0);
}

//-----------------------------------------------------------------------------
// void hal_interrupt_mask(int InterruptNumber)
//-----------------------------------------------------------------------------
// Disable the interrupt indicated by InterruptNumber
//
void hal_interrupt_mask(int InterruptNumber)
{
    if (InterruptNumber<32)
    {
        //
        // Mask the interrupt
        //
        IntControl[0]->VICIntEnClear.Value = (0x1 << InterruptNumber);
    }
    else
    {
        //
        // Mask the interrupt
        //
        IntControl[1]->VICIntEnClear.Value = (0x1 << (InterruptNumber - 32));
    }
}

//-----------------------------------------------------------------------------
// void hal_interrupt_unmask(int InterruptNumber)
//-----------------------------------------------------------------------------
// Enable the interrupt indicated by InterruptNumber
//
void hal_interrupt_unmask(int InterruptNumber)
{
    if (InterruptNumber<32)
    {
        //
        // Mask the interrupt
        //
        IntControl[0]->VICIntEnable.Value = (0x1 << InterruptNumber);
    }
    else
    {
        //
        // Mask the interrupt
        //
        IntControl[1]->VICIntEnable.Value = (0x1 << (InterruptNumber - 32));
    }
}

//-----------------------------------------------------------------------------
// void hal_interrupt_acknowledge(int vector)
//-----------------------------------------------------------------------------
// There is no acknowledge at the global level in the ARM VIC.
//
void hal_interrupt_acknowledge(int vector)
{
    return;
}

//-----------------------------------------------------------------------------
// void hal_interrupt_acknowledge(int vector)
//-----------------------------------------------------------------------------
// All interrupts are level triggered, so this function cannot be implemented.
//
void hal_interrupt_configure(int vector, int level, int up)
{
    return;
}

//-----------------------------------------------------------------------------
// void hal_interrupt_acknowledge(int vector)
//-----------------------------------------------------------------------------
// This function is unimplemented because unless vectored interrupts are used,
// priority is fixed.
//
void hal_interrupt_set_level(int vector, int level)
{
    return;
}

//-----------------------------------------------------------------------------
// cyg_bool ts7250_jp_enabled(int num)
//-----------------------------------------------------------------------------
cyg_bool ts7250_jp_enabled(int num)
{
    unsigned int jp;
    cyg_bool res;

    switch (num) 
    {
    	case 2:
        {
            HAL_READ_UINT8(TS7250_STATUS, jp);
            res = ((jp & 0x1) == 0x1);
	    break;
        }
    	case 3:
        {
            HAL_READ_UINT8(TS7250_STATUS, jp);
            res = ((jp & 0x2) == 0x2);
	    break;
        }
    	case 4:
        {
            HAL_READ_UINT8(TS7250_STATUS, jp);
            res = ((jp & 0x8) != 0x8);
	    break;
        }
    	case 5:
        {
            HAL_READ_UINT8(TS7250_STATUS, jp);
            res = ((jp & 0x10) != 0x10);
	    break;
        }
    	case 6:
        {
            HAL_READ_UINT16(TS7250_JP6, jp);
            res = ((jp & 0x1) == 0x1);
	    break;
        }
    }
    return res;
		
}

int ts7250_ram_present(unsigned long addr)
{
	unsigned int a, b;

	HAL_WRITE_UINT32(addr, 0xaaaaaaaa);
	HAL_WRITE_UINT32(addr + 4, 0x55555555);
	HAL_READ_UINT32(addr, a);
	HAL_READ_UINT32(addr + 4, b);
	if (a == 0xaaaaaaaa && b == 0x55555555) {
		HAL_WRITE_UINT32(addr, 0x55555555);
		HAL_WRITE_UINT32(addr + 4, 0xaaaaaaaa);
	} else {
		return 0;
	}

	HAL_READ_UINT32(addr, a);
	HAL_READ_UINT32(addr + 4, b);
	if (a == 0x55555555 && b == 0xaaaaaaaa) {
		HAL_WRITE_UINT32(addr, 0xaaaaaaaa);
		HAL_WRITE_UINT32(addr + 0x8000000, 0x55555555);
	} else {
		return 0;
	}

	HAL_READ_UINT32(addr, a);
	HAL_READ_UINT32(addr + 0x8000000, b);
	if (a == 0xaaaaaaaa && b == 0x55555555) {
		return 2;
	} else {
		return 1;
	}

	return 0;

}
// ------------------------------------------------------------------------
// EOF ts7250_misc.c
